package org.openxma.dsl.generator.helper.style;

import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.EcoreUtil2;
import org.openxma.dsl.core.CoreFactory;
import org.openxma.dsl.core.model.BoolLiteral;
import org.openxma.dsl.core.model.FalseLiteral;
import org.openxma.dsl.core.model.TrueLiteral;
import org.openxma.dsl.generator.helper.ExpressionExtension;
import org.openxma.dsl.generator.helper.PresentationModelExtension;
import org.openxma.dsl.pom.PomFactory;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.IElementWithLayoutData;
import org.openxma.dsl.pom.model.PaddingWidth;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.StylePropertyFlag;
import org.openxma.dsl.pom.model.StylePropertyOther;
import org.openxma.dsl.pom.model.StylePropertyWidgetVariant;
import org.openxma.dsl.pom.model.StyleSpecificationProperty;
import org.openxma.dsl.pom.model.TabableFlag;
import org.openxma.dsl.pom.util.StyleAccess;

import at.spardat.xma.guidesign.BcdValidator;
import at.spardat.xma.guidesign.IFormaterAttachable;
import at.spardat.xma.guidesign.ValidInState;
import at.spardat.xma.guidesign.XMATableColumn;
import at.spardat.xma.guidesign.XMAWidget;
import at.spardat.xma.guidesign.flex.CompoundValidator;
import at.spardat.xma.guidesign.flex.Constraint;
import at.spardat.xma.guidesign.types.EmptyBoolean;

public class ApplyCombinedStyles {
    
    /**
     * Apply the previously combined styles to the model element. In general this is done in case
     * a property is not already set on the model element.
     * 
     * Sometimes styles do something else (usually more) then setting properties of presentation model
     * elements. These styles usually can not be applied here. To apply them e.g. a access to the 
     * platform specific model (.xma) may be required.
     * 
     * @param obj The presentation model element for which the style will be applied.
     * @param styleProperties All available style properties contained in a map.
     */
    public static void applyToPresentationModelObject(EObject obj, Map<Integer, StyleSpecificationProperty> styleProperties) {
        
        /* Page */
        if (obj instanceof Page) {
            Page page = (Page)obj;                  
            StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);      
            if (stylePropertyOther != null) {
                if (page.getComposeLayout() == null) {                         
                    page.setComposeLayout(createComposeDataOnDemand(styleProperties));
                    applyToPresentationModelObject(page.getComposeLayout(),styleProperties);                    
                }
            }        
        }        
        
        /* Content */
        if (obj instanceof Content) {
            Content content = (Content)obj;                  
            StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);      
            if (stylePropertyOther != null) {
                if (content.getComposeLayout() == null) {                         
                    content.setComposeLayout(createComposeDataOnDemand(styleProperties));
                    applyToPresentationModelObject(content.getComposeLayout(),styleProperties);
                }
            }            
        }
        
        /* ComposeData */
        if (obj instanceof ComposeData) {            
            ComposeData composeData = (ComposeData)obj;            
            StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);
            if (stylePropertyOther != null) { 
                if (composeData.getMargin() == null && stylePropertyOther.getMargin() != null) {
                    composeData.setMargin((PaddingWidth)EcoreUtil2.copy(stylePropertyOther.getMargin()));                                   
                }   
                if (composeData.getSpacing() == null && stylePropertyOther.getSpacing() != null) {
                    composeData.setSpacing((PaddingWidth)EcoreUtil2.copy(stylePropertyOther.getSpacing()));
                }                   
            }
        }
        
        /* IElementWithLayoutData */
        if (obj instanceof IElementWithLayoutData) {
            IElementWithLayoutData iElementWithLayoutData = (IElementWithLayoutData)obj;
            StylePropertyFlag stylePropertyFlag = PresentationModelExtension.getStylePropertyFlag(styleProperties);
            if (iElementWithLayoutData.getTabableFlag() == null && stylePropertyFlag != null && stylePropertyFlag.getTabableExpression() != null) {
                Boolean isTabable = ExpressionExtension.booleanLiteralFor(stylePropertyFlag.getTabableExpression());
                if (isTabable == null) {
                    throw new RuntimeException("Unsupported expression for the style 'tabable' can not be applied to element: "+obj);
                }
                TabableFlag tabableFlag = PomFactory.eINSTANCE.createTabableFlag();
                BoolLiteral value;                
                if (isTabable) {
                    value = CoreFactory.eINSTANCE.createTrueLiteral();    
                } else {
                    value = CoreFactory.eINSTANCE.createFalseLiteral();
                }
                tabableFlag.setValue(value);
                iElementWithLayoutData.setTabableFlag(tabableFlag);
            } 
        }
    }
    
    /**
     * Apply the previously combined styles to the guidesign model element. In general this is done in case
     * a property is not already set on the model element.
     * 
     * @param obj The guidesign model element for which the style will be applied.
     * @param styleProperties All available style properties contained in a map.
     */    
    public static void applyToGuidesignModelObject(EObject obj, Map<Integer, StyleSpecificationProperty> styleProperties) {
        
        /* XMAWidget */
        if (obj instanceof XMAWidget) {
            XMAWidget widget = (XMAWidget)obj;                  
            /* Validator styles */            
            StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);
            if (stylePropertyOther != null && stylePropertyOther.getThousandSeperator() != null) {
                if (widget instanceof IFormaterAttachable) {
                    IFormaterAttachable iFormaterAttachable = (IFormaterAttachable)widget;
                    applyValidatorStyles(iFormaterAttachable.getBdValidator(),stylePropertyOther);                    
                    List<ValidInState> validInStates = (List<ValidInState>)iFormaterAttachable.getValidator();
                    for (ValidInState validInState: validInStates) {
                        at.spardat.xma.guidesign.Validator validator = validInState.getValidator();
                        if (validator instanceof CompoundValidator) {
                            CompoundValidator cv = (CompoundValidator)validator;
                            applyValidatorStyles(cv.getFormatter(),stylePropertyOther);
                            for (Constraint constraint : (List<Constraint>)cv.getConstraints()) {                                    
                                applyValidatorStyles(constraint.getValidator(),stylePropertyOther);
                            }
                        } else {
                            applyValidatorStyles(validator,stylePropertyOther);
                        }                            
                    }                 
                }
            }
            /* Widget variant */
            StylePropertyWidgetVariant stylePropertyWidgetVariant = PresentationModelExtension.getStylePropertyWidgetVariant(styleProperties);
            if (stylePropertyWidgetVariant != null) {
                widget.setWidgetVariant(stylePropertyWidgetVariant.getName());
            }
        }  
        
        /* XMATableColumn */
        if (obj instanceof XMATableColumn) {
            XMATableColumn xmaTableColumn = (XMATableColumn)obj;
            StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);
            if (stylePropertyOther != null && stylePropertyOther.getThousandSeperator() != null) {            
                applyValidatorStyles(xmaTableColumn.getBdValidator(),stylePropertyOther);
            }            
        }
    }

    private static void applyValidatorStyles(at.spardat.xma.guidesign.Validator validator, StylePropertyOther stylePropertyOther) {
        /* Validator not defined */
        if (validator == null) {
            return;
        }
        /* BcdValidator */
        if (validator instanceof BcdValidator) {
            BcdValidator bcdValidator = (BcdValidator)validator;
            if (bcdValidator.getYnThousandSep().getValue() == EmptyBoolean.NOT_SET) {
                if (stylePropertyOther.getThousandSeperator() instanceof TrueLiteral) {
                    bcdValidator.setYnThousandSep(EmptyBoolean.TRUE_LITERAL);
                } else if (stylePropertyOther.getThousandSeperator() instanceof FalseLiteral) {
                    bcdValidator.setYnThousandSep(EmptyBoolean.FALSE_LITERAL);
                }                
            }            
        }      
    }

    /**
     * Create ComposeData in case that there are styles to set.
     * 
     * @param styleProperties
     * @return
     */    
    private static ComposeData createComposeDataOnDemand(Map<Integer, StyleSpecificationProperty> styleProperties) {
        StylePropertyOther stylePropertyOther = PresentationModelExtension.getStylePropertyOther(styleProperties);        
        if (stylePropertyOther != null && (stylePropertyOther.getMargin() != null || stylePropertyOther.getSpacing() != null)) {            
            // Create the compositeLayout (because there are properties to set from styles)                                        
            ComposeData composeData = PomFactory.eINSTANCE.createComposeData();
            StyleAccess.setCombinedStyleProperty(composeData, styleProperties);
            return composeData;
        }
        return null;
        
    }    

}
